import pandas as pd

EXPOSURE_ID = "Exposure.Id"
EXPOSURE_CHANNEL = "Exposure.Channel"
EXPOSURE_TIME = "Exposure.Time"

CLASSIC_CALIBRATION_EXPOSURES = 4

EXPOSURE_COLUMNS = [
    EXPOSURE_ID,
    EXPOSURE_CHANNEL,
    EXPOSURE_TIME,
]

POS_TO_IGNORE = [
    1,
    2,
    9,
    10,
    11,
    12,
    19,
    20,
    71,
    72,
    81,
    82,
    89,
    90,
    91,
    92,
    99,
    100,
]

SIGNAL_SOURCE = "Signal.Source"
SIGNAL_INTENSITY = "Signal.Intensity"
SPOT_MEDIAN = "Spot.Median"
BKG_MEDIAN = "Bkg.Median"
SIGNAL_COLUMNS = [SPOT_MEDIAN, BKG_MEDIAN]


def add_exposure_info(data: pd.DataFrame) -> pd.DataFrame:
    if len(data[EXPOSURE_ID].unique()) == CLASSIC_CALIBRATION_EXPOSURES:
        exposure_values = [
            (1, "Cy3", 100),
            (2, "Cy3", 10),
            (3, "Cy5", 60),
            (4, "Cy5", 6),
        ]
    else:
        exposure_values = [
            (1, "Cy3", 100),
            (2, "Cy3", 30),
            (3, "Cy3", 10),
            (4, "Cy5", 60),
            (5, "Cy5", 18),
            (6, "Cy5", 6),
        ]
    exposure_df = pd.DataFrame(exposure_values, columns=EXPOSURE_COLUMNS)
    return data.merge(exposure_df, on=EXPOSURE_ID)


def remove_non_signal_positions(data: pd.DataFrame) -> pd.DataFrame:
    selection = data["Pos.Id"].isin(POS_TO_IGNORE)
    return data.loc[~selection].copy()


def add_signal_columns(data: pd.DataFrame) -> pd.DataFrame:
    def _generate_signal_columns():
        for column in SIGNAL_COLUMNS:
            copied = data.copy()
            copied[SIGNAL_SOURCE] = column
            copied[SIGNAL_INTENSITY] = data[column]
            yield copied

    return pd.concat(_generate_signal_columns()).reset_index(drop=True)


def prepare_calibration_data(data: pd.DataFrame) -> pd.DataFrame:
    with_exposures = add_exposure_info(data)
    only_signals = remove_non_signal_positions(with_exposures)
    return add_signal_columns(only_signals)
