from __future__ import annotations

import dataclasses

import pandas as pd
from sklearn import linear_model


@dataclasses.dataclass
class Regression:
    intercept: float
    coefficient: float
    score: float

    @property
    def coeff(self) -> float:
        return self.coefficient

    @property
    def r2(self) -> float:
        return self.score

    def predict(
        self, *, x: float | None = None, y: float | None = None
    ) -> float:
        """predict a value if x or y is given"""
        if x is not None and y is not None:
            msg = "predict() expects one keyword argument 'x' or 'y', got both"
            raise TypeError(msg)
        if x is not None:
            return self.intercept + x * self.coefficient
        if y is not None:
            return (y - self.intercept) / self.coefficient
        msg = "predict() expects a keyword argument 'x' or 'y'"
        raise TypeError(msg)

    def to_dict(self):
        return dataclasses.asdict(self)


def linear_regression(data: pd.DataFrame, *, x: str, y: str) -> Regression:
    """calculates a linear regression for two columns of a DataFrame"""
    x_values = data[x].to_numpy().reshape(-1, 1)
    y_values = data[y].to_numpy().reshape(-1, 1)
    fit = linear_model.LinearRegression().fit(x_values, y_values)
    score = fit.score(x_values, y_values)
    return Regression(fit.intercept_[0], fit.coef_[0][0], score)
